'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _objectWithoutProperties = _interopDefault(require('@babel/runtime/helpers/objectWithoutProperties'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var messageformatParser = require('messageformat-parser');
var _slicedToArray = _interopDefault(require('@babel/runtime/helpers/slicedToArray'));
var plurals = _interopDefault(require('make-plural/umd/plurals'));

var isString = function isString(s) {
  return typeof s === "string";
};
var isFunction = function isFunction(f) {
  return typeof f === "function";
};
function isEmpty(obj) {
  // null and undefined are "empty"
  if (obj === null || obj === undefined) return true;
  if (obj.length > 0) return false;
  if (obj.length === 0) return true;
  return !Object.getOwnPropertyNames(obj).length;
}

function date(locales) {
  var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var formatter = new Intl.DateTimeFormat(locales, format);
  return function (value) {
    if (isString(value)) value = new Date(value);
    return formatter.format(value);
  };
}
function number(locales) {
  var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var formatter = new Intl.NumberFormat(locales, format);
  return function (value) {
    return formatter.format(value);
  };
}

var formats = /*#__PURE__*/Object.freeze({
  date: date,
  number: number
});

var defaultFormats = function defaultFormats(locale, locales) {
  var localeData = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
    plurals: undefined
  };
  var formats = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  locales = locales || locale;
  var plurals = localeData.plurals;

  var style = function style(format) {
    return isString(format) ? formats[format] || {
      style: format
    } : format;
  };

  var replaceOctothorpe = function replaceOctothorpe(value, message) {
    return function (ctx) {
      var msg = isFunction(message) ? message(ctx) : message;
      var norm = Array.isArray(msg) ? msg : [msg];
      var formatter = new Intl.NumberFormat(locales);
      var valueStr = formatter.format(value);
      return norm.map(function (m) {
        return isString(m) ? m.replace("#", valueStr) : m;
      });
    };
  };

  return {
    plural: function plural(value, _ref) {
      var _ref$offset = _ref.offset,
          offset = _ref$offset === void 0 ? 0 : _ref$offset,
          rules = _objectWithoutProperties(_ref, ["offset"]);

      var message = rules[value] || rules[plurals(value - offset)];
      return replaceOctothorpe(value - offset, message);
    },
    selectordinal: function selectordinal(value, _ref2) {
      var _ref2$offset = _ref2.offset,
          offset = _ref2$offset === void 0 ? 0 : _ref2$offset,
          rules = _objectWithoutProperties(_ref2, ["offset"]);

      var message = rules[value] || rules[plurals(value - offset, true)];
      return replaceOctothorpe(value - offset, message);
    },
    select: function select(value, rules) {
      return rules[value] || rules.other;
    },
    number: function number$1(value, format) {
      return number(locales, style(format))(value);
    },
    date: function date$1(value, format) {
      return date(locales, style(format))(value);
    },
    undefined: function undefined$1(value) {
      return value;
    }
  };
}; // Params -> CTX

/**
 * Creates a context object, which formats ICU MessageFormat arguments based on
 * argument type.
 *
 * @param locale     - Locale of message
 * @param locales      - Locales to be used when formatting the numbers or dates
 * @param values       - Parameters for variable interpolation
 * @param localeData - Locale data (e.g: plurals)
 * @param formats - Custom format styles
 * @returns {function(string, string, any)}
 */


function context(_ref3) {
  var locale = _ref3.locale,
      locales = _ref3.locales,
      values = _ref3.values,
      formats = _ref3.formats,
      localeData = _ref3.localeData;
  var formatters = defaultFormats(locale, locales, localeData, formats);

  var ctx = function ctx(name, type, format) {
    var value = values[name];
    var formatted = formatters[type](value, format);
    var message = isFunction(formatted) ? formatted(ctx) : formatted;
    return Array.isArray(message) ? message.join("") : message;
  };

  return ctx;
}

function interpolate(translation, locale, locales, localeData) {
  return function (values) {
    var formats = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var message = translation(context({
      locale: locale,
      locales: locales,
      localeData: localeData,
      formats: formats,
      values: values
    }));
    return Array.isArray(message) ? message.join("").trim() : message;
  };
}

function processTokens(tokens) {
  if (!tokens.filter(function (token) {
    return !isString(token);
  }).length) {
    return tokens.join("");
  }

  return function (ctx) {
    return tokens.map(function (token) {
      if (isString(token)) {
        return token; // # in plural case
      } else if (token.type === "octothorpe") {
        return "#"; // simple argument
      } else if (token.type === "argument") {
        return ctx(token.arg); // argument with custom format (date, number)
      } else if (token.type === "function") {
        return ctx(token.arg, token.key, token.params[0]);
      }

      var offset = token.offset ? parseInt(token.offset) : undefined; // complex argument with cases

      var formatProps = {};
      token.cases.forEach(function (item) {
        formatProps[item.key] = processTokens(item.tokens);
      });
      return ctx(token.arg, token.type, _objectSpread({
        offset: offset
      }, formatProps));
    });
  };
} // Message -> (Params -> String)


function compile(message) {
  try {
    return processTokens(messageformatParser.parse(message));
  } catch (e) {
    console.error("Message cannot be parsed due to syntax erorrs: ".concat(message));
    return message;
  }
}

var loadLocaleData = (function (locale) {
  if (!locale) {
    return;
  }

  var _locale$split = locale.split(/[_-]/),
      _locale$split2 = _slicedToArray(_locale$split, 1),
      language = _locale$split2[0];

  return {
    plurals: plurals[language]
  };
});



var icu = /*#__PURE__*/Object.freeze({
  compile: compile,
  loadLocaleData: loadLocaleData
});

var I18n =
/*#__PURE__*/
function () {
  function I18n() {
    _classCallCheck(this, I18n);

    // Messages and localeData are merged on load,
    // so we must initialize it manually
    this._catalogs = {};
    this._didActivate = [];
    this._willActivate = [];
  }

  _createClass(I18n, [{
    key: "loadAll",
    value: function loadAll(catalogs) {
      var _this = this;

      Object.keys(catalogs).forEach(function (locale) {
        return _this.load(locale, catalogs[locale]);
      });
    }
  }, {
    key: "load",
    value: function load(locale, catalog) {
      var _this2 = this;

      if (catalog == null) {
        return Promise.all(this._willActivate.map(function (load) {
          return load(locale).then(function (catalog) {
            return _this2.load(locale, catalog);
          });
        })).then(function () {});
      }

      if (this._catalogs[locale] == null) {
        this._catalogs[locale] = {
          messages: {},
          localeData: {}
        };
      }

      var prev = this._catalogs[locale];
      Object.assign(prev.messages, catalog.messages);
      Object.assign(prev.localeData, catalog.localeData);
      return Promise.resolve();
    }
  }, {
    key: "activate",
    value: function activate(locale, locales) {
      var _this3 = this;

      if (!this._catalogs[locale]) {
        if (this._willActivate.length) {
          return this.load(locale).then(function () {
            return _this3.activate(locale, locales);
          });
        }

        {
          console.warn("Message catalog for locale \"".concat(locale, "\" not loaded."));
        }
      }

      this._locale = locale;
      this._locales = locales;

      this._didActivate.forEach(function (s) {
        return s();
      });

      return Promise.resolve();
    }
  }, {
    key: "use",
    value: function use(locale, locales) {
      var i18n = setupI18n();
      i18n._catalogs = this._catalogs;
      i18n.activate(locale, locales);
      return i18n;
    }
  }, {
    key: "willActivate",
    value: function willActivate(callback) {
      var _this4 = this;

      this._willActivate.push(callback);

      return function () {
        return _this4._willActivate.filter(function (cb) {
          return cb !== callback;
        });
      };
    }
  }, {
    key: "didActivate",
    value: function didActivate(callback) {
      var _this5 = this;

      this._didActivate.push(callback);

      return function () {
        return _this5._didActivate.filter(function (cb) {
          return cb !== callback;
        });
      };
    } // default translate method

  }, {
    key: "_",
    value: function _(id) {
      var values = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
          message = _ref.message,
          formats = _ref.formats;

      if (!isString(id)) {
        values = id.values || values;
        message = id.message;
        id = id.id;
      }

      var translation = this.messages[id] || message || id; // replace missing messages with custom message for debugging

      var missing = this._missing;

      if (missing && !this.messages[id]) {
        return isFunction(missing) ? missing(this.locale, id) : missing;
      }

      {
        translation = isString(translation) ? this.messageFormat.compile(translation) : translation;
      }

      if (!isFunction(translation)) return translation;
      return interpolate(translation, this.locale, this.locales, this.localeData)(values, formats);
    }
  }, {
    key: "date",
    value: function date$1(value, format) {
      return date(this.locales || this.locale, format)(value);
    }
  }, {
    key: "number",
    value: function number$1(value, format) {
      return number(this.locales || this.locale, format)(value);
    }
  }, {
    key: "locale",
    get: function get() {
      return this._locale;
    }
  }, {
    key: "locales",
    get: function get() {
      return this._locales;
    }
  }, {
    key: "catalog",
    get: function get() {
      return this._catalogs[this.locale];
    }
  }, {
    key: "messages",
    get: function get() {
      return this.catalog && this.catalog.messages ? this.catalog.messages : {};
    }
  }, {
    key: "localeData",
    get: function get() {
      {
        if (!this.catalog) {
          this._catalogs[this.locale] = {
            messages: {}
          };
        }

        if (isEmpty(this.catalog.localeData)) {
          this.catalog.localeData = this.messageFormat.loadLocaleData(this._locale);
        }
      }

      return this.catalog.localeData;
    }
  }]);

  return I18n;
}();

function setupI18n() {
  var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var i18n = new I18n();
  i18n.messageFormat = icu;
  if (params.catalogs) i18n.loadAll(params.catalogs);
  if (params.locale) i18n.activate(params.locale, params.locales);
  if (params.missing) i18n._missing = params.missing;
  return i18n;
}

var Messages =
/*#__PURE__*/
function () {
  _createClass(Messages, null, [{
    key: "from",
    value: function from(messages) {
      return new Messages(messages);
    }
  }]);

  function Messages(messages) {
    var _this = this;

    _classCallCheck(this, Messages);

    var _loop = function _loop() {
      var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
          key = _Object$entries$_i[0],
          descriptor = _Object$entries$_i[1];

      if (key === "bind" || key === "i18n") {
        throw new Error("Don't use bind and i18n properties.");
      }

      Object.defineProperty(_this, key, {
        get: function get() {
          return function (values) {
            return _this.i18n._(descriptor, values);
          };
        }
      });
    };

    for (var _i = 0, _Object$entries = Object.entries(messages); _i < _Object$entries.length; _i++) {
      _loop();
    }
  }

  _createClass(Messages, [{
    key: "bind",
    value: function bind(i18n) {
      this.i18n = i18n;
    }
  }]);

  return Messages;
}();

exports.I18n = I18n;
exports.Messages = Messages;
exports.formats = formats;
exports.setupI18n = setupI18n;
