import * as icu from "./dev";
import { MessageDescriptor } from "./messages";
export declare type MessageOptions = {
    message?: string;
    formats?: Object;
};
export declare type Locale = string;
export declare type Locales = Locale | Locale[];
export declare type LocaleData = {
    plurals?: Function;
};
export declare type Messages = {
    [msgId: string]: string | Function;
};
export declare type Catalog = {
    messages: Messages;
    localeData?: LocaleData;
};
export declare type Catalogs = {
    [locale: string]: Catalog;
};
declare type setupI18nProps = {
    locale?: Locale;
    locales?: Locales;
    catalogs?: Catalogs;
    missing?: string | ((message: any, id: any) => string);
};
export declare class I18n {
    messageFormat: typeof icu;
    _catalogs: Catalogs;
    _didActivate: Array<Function>;
    _willActivate: Array<Function>;
    _locale: Locale;
    _locales: Locales;
    _missing: string | ((message: any, id: any) => string);
    constructor();
    readonly locale: string;
    readonly locales: Locales;
    readonly catalog: Catalog;
    readonly messages: Messages;
    readonly localeData: LocaleData;
    loadAll(catalogs: Catalogs): void;
    load(locale: Locale, catalog?: Catalog): Promise<void>;
    activate(locale: Locale, locales?: Locales): any;
    use(locale: Locale, locales?: Locales): I18n;
    willActivate(callback: (locale: Locale) => Promise<Catalog>): Function;
    didActivate(callback: () => void): Function;
    _(id: MessageDescriptor | string, values?: Object | undefined, { message, formats }?: MessageOptions | undefined): any;
    date(value: string | Date, format: Intl.DateTimeFormatOptions): string;
    number(value: number, format: Intl.NumberFormatOptions): string;
}
declare function setupI18n(params?: setupI18nProps): I18n;
export { setupI18n };
//# sourceMappingURL=i18n.d.ts.map